#include "Python.h"
#include <ggi/internal/gii.h>
#include <ggi/errors.h>
#include <ggi/gii.h>

typedef struct {
	PyObject_HEAD
	gii_input_t inp;
} Input;

typedef struct {
	PyObject_HEAD
	gii_event ev;
	PyObject* dict;
} Event;

typedef struct {
	PyTypeObject * InputType;
	PyTypeObject * EventType;
	PyObject* (*newEventFrom)(gii_event e);
	PyObject* (*getDeviceInfo)(struct gii_input *);
	void      (*raiseTimeout)(void);
} pygii_C_api;

#ifdef PYGII_MODULE

extern PyTypeObject InputType;
extern PyTypeObject EventType;

extern void       pygii_raiseTimeout(void);
extern PyObject * pygii_getDeviceInfo(struct gii_input *);
extern PyObject * pygii_newEventFrom(gii_event ev);

extern int        pygii_getInt(PyObject *, int *);
extern int        pygii_getTime(PyObject *, struct timeval *);

#  define EventCheck(op) ((op)->ob_type == &EventType)
#  define InputCheck(op) ((op)->ob_type == &InputType)

#else

extern pygii_C_api * gii_c_api;

#  define EventCheck(op) ((op)->ob_type  == gii_c_api->EventType)
#  define InputCheck(op) ((op)->ob_type  == gii_c_api->InputType)

#endif

/* helpers */

#define ASSERT_EVENT(o,r) do { if(!EventCheck(o)) { \
	       PyErr_SetString(PyExc_TypeError,"Event expected"); \
	       return r; } } while(0)
#define ASSERT_INPUT(o,r) do { if(!InputCheck(o)) { \
	       PyErr_SetString(PyExc_TypeError,"Input expected"); \
	       return r; } } while(0)
#define ASSERT_VALID_INPUT(i,r) do { if(i->inp==NULL) { \
	       PyErr_SetString(PyExc_TypeError,"invalid Input"); \
	       return r; } } while(0)

#define TS_TO_FLOAT(t) \
        ((double)((t)->tv_sec)+((t)->tv_usec/(double)1000000.0))
#define FLOAT_TO_TS(t,f) \
        do { (t)->tv_sec=(int)(f); \
                (t)->tv_usec=(int)(((f) - (int)(f))*(double)1000000.0); \
        } while (0)

